import pandas as pd

# Paths to your input files
states_file = 'inputs/states_objects.csv'
alpha2_file = 'inputs/alpha2.csv'

# Load state codes from states_objects.csv
states_df = pd.read_csv(states_file, header=None, names=['geometry', 'state_code'])
state_codes = states_df['state_code'].unique().tolist()

# Load country codes from alpha2.csv
alpha2_df = pd.read_csv(alpha2_file)

# Normalized location names from your previous script (excluding 'UnitedStates')
interested_locations = ['Austria', 'Belgium', 'Bulgaria', 'Croatia', 'Cyprus', 'CzechRepublic', 'Denmark', 'Estonia', 'Finland', 'France', 'Germany', 'Greece', 'Hungary', 'Ireland', 'Italy', 'Latvia', 'Lithuania', 'Luxembourg', 'Malta', 'Netherlands', 'Poland', 'Portugal', 'Romania', 'Slovakia', 'Slovenia', 'Spain', 'Sweden', 'Iceland', 'Switzerland', 'Liechtenstein', 'Norway', 'UnitedKingdom', 'Ukraine']

# Normalize location names
location_map = {
    'CzechRepublic': 'Czech Republic',
    'UnitedKingdom': 'United Kingdom',
    # Add more mappings if necessary
}
normalized_locations = [location_map.get(loc, loc) for loc in interested_locations]

# Map normalized names to codes
location_code_map = dict(zip(alpha2_df['Name'], alpha2_df['Code']))
country_codes = [location_code_map.get(loc) for loc in normalized_locations if location_code_map.get(loc)]

# Exclude 'US' from country codes
country_codes = [code for code in country_codes if code != 'US']

# Combine all id values
id_values = state_codes + country_codes + ['']  # Add empty id

# Remove duplicates
id_values = list(set(id_values))

# Generate partition definitions
partition_definitions = []
for id_value in sorted(id_values):
    if id_value == '':
        partition_name = 'p_unknown'
        partition_definitions.append(f"    PARTITION {partition_name} VALUES IN ('')")
    else:
        partition_name = f"p_{id_value}"
        partition_definitions.append(f"    PARTITION {partition_name} VALUES IN ('{id_value}')")

# Write the partition_db.sql file
with open('inputs/sql/partition_db.sql', 'w') as f:
    f.write('ALTER TABLE FACT_bldgs.footprints\nPARTITION BY LIST COLUMNS (id) (\n')
    f.write(',\n'.join(partition_definitions))
    f.write('\n);\n')
