import os
import pandas as pd
import geopandas as gpd
from shapely.geometry import shape
from concurrent.futures import ThreadPoolExecutor

# Define the list of locations you're interested in
interested_locations = ['Austria', 'Belgium', 'Bulgaria', 'Croatia', 'Cyprus', 'CzechRepublic', 'Denmark', 'Estonia', 'Finland', 'France', 'Germany', 'Greece', 'Hungary', 'Ireland', 'Italy', 'Latvia', 'Lithuania', 'Luxembourg', 'Malta', 'Netherlands', 'Poland', 'Portugal', 'Romania', 'Slovakia', 'Slovenia', 'Spain', 'Sweden', 'Iceland', 'Switzerland', 'Liechtenstein', 'Norway', 'UnitedKingdom', 'Ukraine', 'UnitedStates']


def normalize_location_name(location):
    # Map location names to match those in alpha2.csv
    location_map = {
        'CzechRepublic': 'Czech Republic',
        'UnitedKingdom': 'United Kingdom',
        # Add more mappings if necessary
    }
    return location_map.get(location, location)

def add_id_field(gdf, location):
    if location == 'UnitedStates':
        # Add state codes to features
                # Load state polygons with state codes from 'states_objects.csv'
        states_df = pd.read_csv('inputs/states_objects.csv', header=None, names=['geometry', 'state_code'])
        # Convert WKT geometries to shapely geometries
        states_df['geometry'] = states_df['geometry'].apply(wkt.loads)
        states_gdf = gpd.GeoDataFrame(states_df, geometry='geometry')
        states_gdf = states_gdf.set_crs(gdf.crs)  # Ensure CRS matches

        # Spatial join
        joined_gdf = gpd.sjoin(gdf, states_gdf[['geometry', 'state_code']], how='left', predicate='intersects')
        # 'state_code' is the two-letter state code
        gdf['id'] = joined_gdf['state_code']

        # Handle features that didn't match any state (if any)
        gdf['id'] = gdf['id'].fillna('Unknown')

    else:
        # For other locations, match the location name to the code from alpha2.csv
        # Normalize the location name to match the names in alpha2.csv
        normalized_location = normalize_location_name(location)

        # Load alpha2.csv
        alpha2_df = pd.read_csv('inputs/alpha2.csv')

        # Create a mapping from country names to codes
        country_code_map = dict(zip(alpha2_df['Name'], alpha2_df['Code']))

        # Match the normalized location name to the code
        code = country_code_map.get(normalized_location, 'Unknown')

        # Add the code to the 'id' field of all features
        gdf['id'] = code

    return gdf

def process_file(row):
    file_path = f"data/{row['QuadKey']}.geojson"
    if row['Location'] in interested_locations and not os.path.exists(file_path):
        df = pd.read_json(row['Url'], lines=True)
        df['geometry'] = df['geometry'].apply(shape)
        gdf = gpd.GeoDataFrame(df, crs=4326)
        # Call the function to add the 'id' field if in UnitedStates
        gdf = add_id_field(gdf, row['Location'])
        gdf.to_file(file_path, driver="GeoJSON")

def main():
    dataset_links = pd.read_csv("https://minedbuildings.blob.core.windows.net/global-buildings/dataset-links.csv")
    if not os.path.exists('data'):
        os.makedirs('data')

    # Use ThreadPoolExecutor to process files in parallel
    with ThreadPoolExecutor(max_workers=20) as executor:
        executor.map(process_file, dataset_links.to_dict(orient='records'))

if __name__ == "__main__":
    main()
